(*-------------------------------------------------------------------------*)
(*                                                                         *)
(*  PASCAL-XSC - MODUL  IEP_RAWA                                 05.05.93  *)
(*                                                                         *)
(*  (c) Markus Neher    Institute for Applied Mathematics                  *)
(*                      D-76128 Karlsruhe University, Germany              *)
(*                      e-mail: ae16@rz.uni-karlsruhe.de                   *)
(*                                                                         *)
(*  REELLER ANFANGSWERTPROBLEMLSER FR DIE STURM-LIOUVILLE-GLEICHUNG      *)
(*                                                                         *)
(*    AUSGANGSPROBLEM :      - U" + P(X)  U(X) =   U(X)                 *)
(*                                                                         *)
(*    TRANSFORMIERT AUF :      U1' = U2                                    *)
(*                             U2' = ( P(X) -  )  U1                     *)
(*                                                                         *)
(*-------------------------------------------------------------------------*)

MODULE iep_rawa;

USE mv_ari , iep_tayk , service , splines;

GLOBAL TYPE RITZTENSOR = GLOBAL DYNAMIC ARRAY[*,*,*] OF REAL;
GLOBAL TYPE RTTENSOR = GLOBAL DYNAMIC ARRAY[*] OF RTMAT;

(*-------------------------------------------------------------------------*)

FUNCTION TAYKOEFF( TRP : RTKO; VRU : RVECTOR ) : RTVEC[1..2,0..UB(TRP)];

VAR NR , P : INTEGER;
    TRVECU : RTVEC[1..2,0..UB(TRP)];

BEGIN P := UB( TRP );
   TRVECU[1] := TKO( P , VRU[1] );
   TRVECU[2] := TKO( P , VRU[2] );
                                               
   FOR NR := 1 TO P DO BEGIN
       (*  U1' := U2  *)
       TRVECU[1,NR] := TRVECU[2,NR-1] / NR;   
       (*  U2' := FCP * U1  *)
       TRVECU[2,NR] := TRPROD( TRP , TRVECU[1] , NR - 1 ) / NR;
   END; (* FOR *)

   TAYKOEFF := TRVECU;
   
END; 

(*-------------------------------------------------------------------------*)

GLOBAL FUNCTION PUNKTAUSWERTUNG( H : REAL; TRU : RTKO ) : REAL;

VAR I , P : INTEGER;
    X     : REAL;

BEGIN P := UB( TRU );

   X := TRU[P-1];
   FOR I := P - 2 DOWNTO 0 DO
       X := H * X + TRU[I];

   PUNKTAUSWERTUNG := X;

END; 

(*-------------------------------------------------------------------------*)

GLOBAL FUNCTION PUNKTAUSWERTUNG( H : REAL; TRVECU : RTVEC ) : 
                                 RVECTOR[ LB( TRVECU ) .. UB( TRVECU ) ];

VAR I   : INTEGER;
    VRU : RVECTOR[ LB( TRVECU ) .. UB( TRVECU ) ];

BEGIN 

   FOR I := LB( TRVECU ) TO UB( TRVECU ) DO
       VRU[I] := PUNKTAUSWERTUNG( H , TRVECU[I] );

   PUNKTAUSWERTUNG := VRU;

END; 

(*-------------------------------------------------------------------------*)

GLOBAL FUNCTION AWA( UPBD , P : INTEGER; H , EIG : REAL; TRVECP : RTVEC; )
                   : RVECTOR[1..2];

(*  AWP-LSER FR DIE STURM-LIOUVILLE-GLEICHUNG  *)

VAR ST     : INTEGER;
    T      : REAL;
    TRP    : RTKO[0..P];
    VRU    : RVECTOR[1..2];
    TRVECU : RTVEC[1..2,0..P];

BEGIN VRU[1] := 0;
   VRU[2] := 1;

   (*  JETZT STARTET DAS LSUNGSVERFAHREN DER DGL  *)
   
   FOR ST := 0 TO UPBD - 1 DO BEGIN 

       TRP := TRVECP[ST];
       TRP[0] := TRP[0] - EIG;
       TRVECU := TAYKOEFF( TRP , VRU );
       VRU := PUNKTAUSWERTUNG( H , TRVECU );
       
       WRITE( 'ST :  .....................................................' );
       WRITELN( '...............' , ST : 4 );
    
   END;

   AWA := VRU;
  
END;         

(*-------------------------------------------------------------------------*)

GLOBAL PROCEDURE AWA_NWT( UPBD , P : INTEGER; H , EIG : REAL;
       TRVECP : RTVEC; VAR TRMATQ : RTMAT;
       VAR VRU : RVECTOR; VAR VR_U2 : REAL; VAR VR_U2Q : RVECTOR );

(*  AWP-LSER FR DIE STURM-LIOUVILLE-GLEICHUNG  *)
(*  NEWTON-VERFAHREN FR SPLINE-BASIS            *)
(*  HIER : REELLE RECHNUNG OHNE EINSCHLIESSUNG ! *)


(*  UPBD     :  ANZAHL DER STTZSTELLEN IM INTERVALL [ 0 ,  ]  *)
(*  P        :  ORDNUNG DER TAYLOR-ENTWICKLUNG                  *)
(*  H        :  SCHRITTWEITE                                    *)
(*  EIG      :  EIGENWERT                                       *)
(*  TRVECP   :  TAYLORKOEFFIZIENTEN DES POTENTIALS P(X)         *)

VAR I , NR , ST , N , J       : INTEGER;
    T                         : REAL;
    TRP , TR_U2 , TR_U2_IGRAL : RTKO[0..P];
    TRVECU                    : RTVEC[1..2,0..P];
    TRVECQ , TRVEC_U2Q_IGRAL  : RTVEC[0..UB(VR_U2Q),0..P];

BEGIN N := UB( VR_U2Q );
   
   VRU[1] := 0;
   VRU[2] := 1;           
   VR_U2 := 0;
   FOR I := 0 TO N DO
       VR_U2Q[I] := 0;

   (*  JETZT STARTET DAS LSUNGSVERFAHREN DER DGL  *)
   
   FOR ST := 0 TO UPBD - 1 DO BEGIN

       TRP := TRVECP[ST];
       TRP[0] := TRP[0] - EIG;
       FOR I := 0 TO N DO
           TRVECQ[I] := TRMATQ[I,ST];

       (*  TAYLOR-KOEFFIZIENTEN VON U BERECHNEN  *)

       TRVECU := TAYKOEFF( TRP , VRU );
       VRU := PUNKTAUSWERTUNG( H , TRVECU );
       
       (*  TAYLOR-KOEFFIZIENTEN VON U^2 UND INTEGRAL DAVON BERECHNEN  *)

       TR_U2_IGRAL[0] := VR_U2;
       FOR NR := 0 TO P - 1 DO BEGIN
           TR_U2[NR] := TRPROD( TRVECU[1] , TRVECU[1] , NR );
           TR_U2_IGRAL[NR+1] := TR_U2[NR] / ( NR + 1 );
       END;

       VR_U2 := PUNKTAUSWERTUNG( H , TR_U2_IGRAL );

       (*  TAYLOR-KOEFFIZIENTEN VON U^2 * Q[I]  (BASIS-SPLINES)  *)
       (*  UND INTEGRAL DAVON BERECHNEN                          *)

       FOR I := 0 TO N DO BEGIN
           TRVEC_U2Q_IGRAL[I,0] := VR_U2Q[I];
           FOR NR := 0 TO P - 1 DO
               TRVEC_U2Q_IGRAL[I,NR+1] := TRPROD(TR_U2,TRVECQ[I],NR) / (NR+1);
       END;    

       VR_U2Q := PUNKTAUSWERTUNG( H , TRVEC_U2Q_IGRAL );

       WRITE( 'ST :  .....................................................' );
       WRITELN( '...............' , ST : 4 );
    
   END; (* FOR , ST *); 

END;

(*-------------------------------------------------------------------------*)

GLOBAL FUNCTION AWPS( SPP : RSPLINE; EIG : REAL ) : REAL;

(*                                                                          *)
(*  ANFANGSWERTL"OSER F"UR STURM-LIOUVILLE-GLEICHUNG MIT SPLINE-POTENTIAL   *)
(*                                                                          *)
(*    AUSGANGSPROBLEM :      - u" + q(x) u =   u                          *)
(*    TRANSFORMIERT AUF :      u" = ( q(x) -  ) u                          *)
(*                                                                          *)
(*  HIER : EINFACHSTE L"OSUNG VON ANFANGSWERTPROBLEMEN                      *)
(*         ENTWICKLUNG VON q(x) IN REIHE                                    *)
(*                                                                          *)
(*         q(x) =  c_0 + c_1 (x-x_i) + c_2 (x-x_i)^2 + c_3 (x-x_i)^3        *)
(*                                                                          *)
(*  POTENZREIHENANSATZ F"UR u(x) F"UHRT AUF REKUSRIONSFORMEL                *)
(*  L"OSUNG MIT VARIABLER TAYLORENTWICKLUNG UND RESTGLIEDABSCH"ATZUNG       *)
(*  IN EINEM EINZIGEN SCHRITT                                               *)

CONST P = 200;            (*  MAXIMALE ORDNUNG DER TAYLORENTWICKLUNG VON U  *)

(*  GLOBALE VARIABLEN :                                           *)
(*                                                                *)
(*     M   : M + 1 = ANZAHL DER ST"UTZSTELLEN DES SPLINES P(X)    *)
(*     N   : N + 1 = ANZAHL DER VORGEGEBENEN EIGENWERTE VON P(X)  *)

VAR EW , M , N , I , J , NR                  : INTEGER;
    MAXA , MAXC , RESTGLIED , ABLRESTGLIED   : REAL;
    PI , H , H2 , H3 , R1 , REIHE , ABLREIHE : REAL;
    TRA                                      : RVECTOR[0..P];
    TRC                                      : RVECTOR[0..3];

BEGIN

   M := UB( SPP , 2 );
   N := M DIV 2;

   (* SCHRITTWEITE SETZEN *)

   PI := RVAL( '3.141592653589793115997963468544185161590576171875' );
   H  := PI / M;
   H2 := SQR( H );
   H3 := H2 * H;

   (*  SCHLEIFE "UBER ALLE ST"UTZSTELLEN DES SPLINES  *)

   (*  ANFANGSWERTE FESTLEGEN  *)

   REIHE := 0;
   ABLREIHE := 1;

   FOR I := 0 TO M - 1 DO BEGIN

       (*  TAYLORKOEFFIZIENTEN VON P(X) BEREITSTELLEN       *)
       (*  BEACHTE, DASS DIE ABLEITUNGEN DES SPLINES        *)
       (*  BEREITS MIT POTENZEN VON h MULTIPLIZIERT WURDEN  *)

       TRC[0] := ( SPP[0,I] - EIG ) * H2;
       TRC[1] := SPP[1,I] * H2;
       TRC[2] := SPP[2,I] * H2 / 2;
       TRC[3] := SPP[3,I] * H2 / 6;

       (*  MINDESTANZAHL DER TERME IN DER TAYLORENTWICKLUNG BERECHNEN  *)

       MAXC := SQRT( ABS( TRC[0] ) + ABS( TRC[1] / H )
                   + ABS( TRC[2] / H2 ) + ABS( TRC[3] / H3 ) );

       (*  ANFANGSWERTE FESTLEGEN  *)

       TRA[0] := REIHE;
       TRA[1] := ABLREIHE;

       (*  "VORITERATION"  *)

       TRA[2] := TRC[0] * TRA[0] / 2;
       TRA[3] := #*( TRC[0] * TRA[1] + TRC[1] * TRA[0] ) / 6;
       TRA[4] := #*( TRC[0] * TRA[2] + TRC[1] * TRA[1]
                   + TRC[2] * TRA[0] ) / 12;

       REIHE := #*( FOR J := 0 TO 4 SUM( TRA[J] ) );
       ABLREIHE := #*( FOR J := 1 TO 4 SUM( J * TRA[J] ) );

       (*  WERTE ZUM EINSPRUNG IN SCHLEIFE VORBESETZEN  *)

       RESTGLIED := REIHE;
       ABLRESTGLIED := ABLREIHE;

       NR := 4;

       WHILE ( ( NR < MAXC ) OR ( ABS( RESTGLIED / REIHE )
             + ABS( ABLRESTGLIED / ABLREIHE ) > 1.0E-16 ) ) DO BEGIN

             NR := NR + 1;

             TRA[NR] := #*( FOR J := 0 TO 3
                                  SUM( TRC[J] * TRA[NR-2-J] ) )
                                       / ( NR * ( NR - 1 ) );

             REIHE := REIHE + TRA[NR];
             ABLREIHE := ABLREIHE + NR * TRA[NR];

             (* RESTGLIED ABSCH"ATZEN  *)
             (* ZUERST MAXIMUM DER LETZTEN F"UNF KOEFFIZIENTEN BESTIMMEN *)

             MAXA := ABS( TRA[NR] );
             FOR J := 1 TO 4 DO
                 IF ( H ** J ) * ( ABS( TRA[NR-J] ) ) > MAXA THEN
                    MAXA := ( H ** J ) * ( ABS( TRA[NR-J] ) );

             R1 := H / ( 1 - H );
             RESTGLIED := MAXA * R1;
             R1 := ( 1 + NR * ( 1 - H ) ) / SQR( 1 - H );
             ABLRESTGLIED := MAXA * R1;

       END;

       WRITELN( 'Restglied [' , NR : 2 , ']      = ' , RESTGLIED );
       WRITELN( 'Ableitungsrestglied = ' , ABLRESTGLIED );
       WRITELN;

       REIHE    := #*( FOR J := 0 TO NR SUM( TRA[J] ) );
       ABLREIHE := #*( FOR J := 1 TO NR SUM( J * TRA[J] ) );

       WRITELN( 'u (' , I + 1 : 2 , '  / ' , M : 1 , ' ) = ' , REIHE );
       WRITELN( 'u''(' , I + 1 : 2 , '  / ' , M : 1 , ' ) = ', ABLREIHE );
       WRITELN;

   END; (* FOR , I *)

   AWPS := REIHE;

END;

(*--------------------------------------------------------------------------*)

GLOBAL PROCEDURE AWPS_NWT( SPP : RSPLINE; EIG : REAL; SPQ : RSVECTOR;
       VAR VRU : RVECTOR; VAR VR_U2 : REAL; VAR VR_U2Q : RVECTOR );

(*                                                                          *)
(*  ANFANGSWERTL"OSER F"UR STURM-LIOUVILLE-GLEICHUNG MIT SPLINE-POTENTIAL   *)
(*                                                                          *)
(*    AUSGANGSPROBLEM :      - u" + q(x) u =   u                          *)
(*    TRANSFORMIERT AUF :      u" = ( q(x) -  ) u                          *)
(*                                                                          *)
(*  HIER : EINFACHSTE L"OSUNG VON ANFANGSWERTPROBLEMEN                      *)
(*         ENTWICKLUNG VON q(x) IN REIHE                                    *)
(*                                                                          *)
(*         q(x) =  c_0 + c_1 (x-x_i) + c_2 (x-x_i)^2 + c_3 (x-x_i)^3        *)
(*                                                                          *)
(*  POTENZREIHENANSATZ F"UR u(x) F"UHRT AUF REKUSRIONSFORMEL                *)
(*  L"OSUNG MIT VARIABLER TAYLORENTWICKLUNG UND RESTGLIEDABSCH"ATZUNG       *)
(*  IN EINEM EINZIGEN SCHRITT                                               *)

CONST P = 200;            (*  MAXIMALE ORDNUNG DER TAYLORENTWICKLUNG VON U  *)

(*  GLOBALE VARIABLEN :                                           *)
(*                                                                *)
(*     M   : M + 1 = ANZAHL DER ST"UTZSTELLEN DES SPLINES P(X)    *)
(*     N   : N + 1 = ANZAHL DER VORGEGEBENEN EIGENWERTE VON P(X)  *)

VAR EW , M , N , I , J , K , L , NR          : INTEGER;
    MAXA , MAXC , RESTGLIED , ABLRESTGLIED   : REAL;
    PI , H , H2 , H3 , R1 , REIHE , ABLREIHE : REAL;
    TRC                                      : RVECTOR[0..3];
    TRA , TRU2 , TRU2INT                     : RVECTOR[0..P];
    TRU2Q                                    : RMATRIX[0..UB(VR_U2Q),0..P];

(* TRA   ENTHLT DIE SUMMANDEN DER POTENZREIHE VON u (MIT POTENZEN VON h^j) *)
(* TRU2  ENTHLT DIE SUMMANDEN VON u^2                                      *)
(* TRU2INT ENTHLT DIE SUMMANDEN VON \int u^2 dx                            *)
(* TRU2Q ENTHLT DIE SUMMANDEN VON \int u^2 q_j dx                          *)

BEGIN

   M := UB( SPP , 2 );
   N := M DIV 2;

   (* SCHRITTWEITE SETZEN *)

   PI := RVAL( '3.141592653589793115997963468544185161590576171875' );
   H  := PI / M;
   H2 := SQR( H );
   H3 := H2 * H;

   (*  SCHLEIFE "UBER ALLE ST"UTZSTELLEN DES SPLINES  *)

   (*  ANFANGSWERTE FESTLEGEN  *)

   REIHE := 0;
   ABLREIHE := 1;
   VR_U2 := 0;
   FOR J := 0 TO N DO
       VR_U2Q[J] := 0;

   FOR I := 0 TO M - 1 DO BEGIN

       (*  TAYLORKOEFFIZIENTEN VON P(X) BEREITSTELLEN       *)
       (*  BEACHTE, DASS DIE ABLEITUNGEN DES SPLINES        *)
       (*  BEREITS MIT POTENZEN VON h MULTIPLIZIERT WURDEN  *)

       TRC[0] := ( SPP[0,I] - EIG ) * H2;
       TRC[1] := SPP[1,I] * H2;
       TRC[2] := SPP[2,I] * H2 / 2;
       TRC[3] := SPP[3,I] * H2 / 6;
       FOR K := 0 TO N DO BEGIN
           SPQ[K,2,I] := SPQ[K,2,I] / 2;
           SPQ[K,3,I] := SPQ[K,3,I] / 6;
       END;

       (*  MINDESTANZAHL DER TERME IN DER TAYLORENTWICKLUNG BERECHNEN  *)

       MAXC := SQRT( ABS( TRC[0] ) + ABS( TRC[1] / H )
                   + ABS( TRC[2] / H2 ) + ABS( TRC[3] / H3 ) );

       (*  ANFANGSWERTE FESTLEGEN  *)

       TRA[0] := REIHE;
       TRA[1] := ABLREIHE;

       (*  "VORITERATION"  *)

       TRA[2] := TRC[0] * TRA[0] / 2;
       TRA[3] := #*( TRC[0] * TRA[1] + TRC[1] * TRA[0] ) / 6;
       TRA[4] := #*( TRC[0] * TRA[2] + TRC[1] * TRA[1]
                   + TRC[2] * TRA[0] ) / 12;

       REIHE := #*( FOR J := 0 TO 4 SUM( TRA[J] ) );
       ABLREIHE := #*( FOR J := 1 TO 4 SUM( J * TRA[J] ) );

       (*  WERTE ZUM EINSPRUNG IN SCHLEIFE VORBESETZEN  *)

       RESTGLIED := REIHE;
       ABLRESTGLIED := ABLREIHE;

       NR := 4;

       WHILE ( ( NR < MAXC ) OR ( ABS( RESTGLIED / REIHE )
             + ABS( ABLRESTGLIED / ABLREIHE ) > 1.0E-16 ) ) DO BEGIN

             NR := NR + 1;

             TRA[NR] := #*( FOR J := 0 TO 3
                                  SUM( TRC[J] * TRA[NR-2-J] ) )
                                       / ( NR * ( NR - 1 ) );

             REIHE := REIHE + TRA[NR];
             ABLREIHE := ABLREIHE + NR * TRA[NR];

             (* RESTGLIED ABSCH"ATZEN  *)
             (* ZUERST MAXIMUM DER LETZTEN F"UNF KOEFFIZIENTEN BESTIMMEN *)

             MAXA := ABS( TRA[NR] );
             FOR J := 1 TO 4 DO
                 IF ( H ** J ) * ( ABS( TRA[NR-J] ) ) > MAXA THEN
                    MAXA := ( H ** J ) * ( ABS( TRA[NR-J] ) );

             R1 := H / ( 1 - H );
             RESTGLIED := MAXA * R1;
             R1 := ( 1 + NR * ( 1 - H ) ) / SQR( 1 - H );
             ABLRESTGLIED := MAXA * R1;

       END;

       (* JETZT REIHEN FR u^2, \int u^2 dx, \int u^2 q_j dx  BERECHNEN *)
       (* ES WERDEN NUR DIE TERME BERCKSICHTIGT, DIE EINEN "GROSSEN"   *)
       (* BEITRAG ZUR REIHE LEISTEN                                     *)

       FOR J := 0 TO NR DO BEGIN

           TRU2[J] := #*( FOR L := 0 TO J SUM( TRA[L] * TRA[J-L] ) );
           TRU2INT[J+1] := H * TRU2[J] / ( J + 1 );
           FOR K := 0 TO N DO
               IF J < 3 THEN
                  TRU2Q[K,J+1] := H * #*( FOR L := 0 TO J
                               SUM( SPQ[K,L,I] * TRU2[J-L] ) ) / ( J + 1 )
               ELSE
                  TRU2Q[K,J+1] := H * #*( FOR L := 0 TO 3
                               SUM( SPQ[K,L,I] * TRU2[J-L] ) ) / ( J + 1 );

       END;

       (*  REIHENGLIEDER SUMMIEREN  *)

       VR_U2 := #*( VR_U2 + FOR J := 1 TO NR + 1 SUM( TRU2INT[J] ) );

       FOR K := 0 TO N DO
           VR_U2Q[K] := #*( VR_U2Q[K] + FOR J := 1 TO NR+1 SUM( TRU2Q[K,J] ) );

       (*  WERTE AUSGEBEN  *)

       WRITELN( 'Restglied [' , NR : 2 , ']      = ' , RESTGLIED );
       WRITELN( 'Ableitungsrestglied = ' , ABLRESTGLIED );
       WRITELN;

       REIHE    := #*( FOR J := 0 TO NR SUM( TRA[J] ) );
       ABLREIHE := #*( FOR J := 1 TO NR SUM( J * TRA[J] ) );

       WRITELN( 'u (' , I + 1 : 2 , '  / ' , M : 1 , ' ) = ' , REIHE );
       WRITELN( 'u''(' , I + 1 : 2 , '  / ' , M : 1 , ' ) = ', ABLREIHE );
       WRITELN;

   END; (* FOR , I *)

   VRU[1] := REIHE;
   VRU[2] := ABLREIHE;

END;

(*--------------------------------------------------------------------------*)

GLOBAL PROCEDURE AWA_RITZ( UPBD , N , P : INTEGER; H : REAL; EIGVAL : RVECTOR;
       TRVECP : RTVEC; VAR TRMATQ : RTMAT; VAR MRU , MRW : RMATRIX;
       VAR VR_U2 : RVECTOR; VAR MR_U2Q : RMATRIX; VAR RITZ_MAT : RITZTENSOR );

(*  SIMULTANER AWP-LSER FR DIE STURM-LIOUVILLE-GLEICHUNG      *)
(*  MIT EIGENFUNKTIONENBASIS ?                                  *)

(*  UPBD     :  ANZAHL DER STTZSTELLEN IM INTERVALL [ 0 ,  ]  *)
(*  N        ;  ANZAHL DER VORGEGEBENEN EIGENWERTE              *)
(*  P        :  ORDNUNG DER TAYLOR-ENTWICKLUNG                  *)
(*  H        :  SCHRITTWEITE                                    *)
(*  TRVECP   :  TAYLORKOEFFIZIENTEN DES POTENTIALS P(X)         *)
(*  TRMATU   :  TAYLORKOEFFIZIENTEN DER EIGENFUNKTIONEN U[I](X) *)

VAR EW , ST , I , J , K , NR : INTEGER;
    FAKTOR                   : REAL;
    VRU , VRW                : RVECTOR[1..2];
    TRP                      : RTKO[0..P];
    TRMATU , TRMATW          : RTMAT[0..N,0..UPBD,0..P];
    TR_UHILFS , TR_WHILFS    : RTVEC[1..2,0..P];
    TR_DUW , TRVECQ          : RTVEC[0..N,0..P];
    TR_U2 , TR_U2_IGRAL      : RTVEC[0..N,0..P];
    TR_UUW , TRMAT_U2Q_IGRAL : RTMAT[0..N,0..N,0..P];
    TR_UUWU , TR_UUWU_IGRAL  : RTTENSOR[0..N,0..N,0..N,0..P];
    UUWU_IGRAL               : RITZTENSOR[0..N,0..N,0..N];
    
BEGIN 
   
   FOR EW := 0 TO N DO BEGIN

       VRU[1] := 0;
       VRU[2] := 1;           
       VRW[1] := 1;
       VRW[2] := 0;           
       VR_U2[EW] := 0;
       FOR I := 0 TO N DO
           MR_U2Q[EW,I] := 0;

       (*  JETZT STARTET DAS LSUNGSVERFAHREN DER DGL  *)
   
       FOR ST := 0 TO UPBD - 1 DO BEGIN

           TRP := TRVECP[ST];
           TRP[0] := TRP[0] - EIGVAL[EW];
           FOR I := 0 TO N DO
               TRVECQ[I] := TRMATQ[I,ST];

           (*  TAYLOR-KOEFFIZIENTEN VON U BERECHNEN  *)

           TR_UHILFS := TAYKOEFF( TRP , VRU );
           TR_WHILFS := TAYKOEFF( TRP , VRW );
           TRMATU[EW,ST] := TR_UHILFS[1];
           TRMATW[EW,ST] := TR_WHILFS[1];
           VRU := PUNKTAUSWERTUNG( H , TR_UHILFS );
           VRW := PUNKTAUSWERTUNG( H , TR_WHILFS );

           (*  TAYLOR-KOEFFIZIENTEN VON U^2 UND INTEGRAL DAVON BERECHNEN  *)

           TR_U2_IGRAL[EW,0] := VR_U2[EW];
           FOR NR := 0 TO P - 1 DO BEGIN
               TR_U2[EW,NR] := TRPROD( TRMATU[EW,ST] , TRMATU[EW,ST], NR );
               TR_U2_IGRAL[EW,NR+1] := TR_U2[EW,NR] / ( NR + 1 );
           END;

           VR_U2[EW] := PUNKTAUSWERTUNG( H , TR_U2_IGRAL[EW] );

           (*  TAYLOR-KOEFFIZIENTEN VON U^2 * Q[I]  (BASIS-SPLINES)  *)
           (*  UND INTEGRAL DAVON BERECHNEN                          *)

           FOR I := 0 TO N DO BEGIN
               TRMAT_U2Q_IGRAL[EW,I,0] := MR_U2Q[EW,I];
               FOR NR := 1 TO P DO 
                   TRMAT_U2Q_IGRAL[EW,I,NR] :=
                              TRPROD( TR_U2[EW], TRMATQ[I,ST], NR - 1 ) / NR;
           END;    

           MR_U2Q[EW] := PUNKTAUSWERTUNG( H , TRMAT_U2Q_IGRAL[EW] );

           WRITE( 'ST :  .................................................' );
           WRITELN( '...................' , ST : 4 );
    
       END; (* FOR , ST *); 

       MRU[EW] := VRU;
       MRW[EW] := VRW;
       
       (*  JETZT :  W[J]  (ANTI-)SYMMETRISIEREN,       *)
       (*  D.H. GEEIGNETE VIELFACHE VON U[J] ADDIEREN  *)
        
       FAKTOR := SIGN( MRU[EW,2] ) * MRW[EW,2];
       FOR ST := 0 TO UPBD - 1 DO
           TRMATW[EW,ST] := TRMATW[EW,ST] - FAKTOR * TRMATU[EW,ST] / 2;
       
   END; (* FOR , EW *); 
    
   (*  JETZT RITZ-TENSOR BERECHNEN                      *)
   (*  RT[I,J,K] = IGRAL( U[I] * U[J] * W[J] * U[K] ),  *)

   FOR J := 0 TO N DO
       FOR I := 0 TO N DO
           FOR K := 0 TO N DO
               UUWU_IGRAL[I,J,K] := 0;

   FOR ST := 0 TO UPBD - 1 DO BEGIN

       FOR J := 0 TO N DO BEGIN
           
           (*  TAYLORKOEFFIZIENTEN VON  ( U[J] * W[J] )'  BERECHNEN  *)
           
           FOR NR := 1 TO P DO 
               TR_DUW[J,NR-1] := NR*TRPROD( TRMATU[J,ST], TRMATW[J,ST], NR );

           FOR I := 0 TO N DO BEGIN

               (*  TAYLORKOEFFIZIENTEN VON  U[I] * (U[J]*W[J])'  BERECHNEN  *)
           
               FOR NR := 0 TO P - 1 DO 
                   TR_UUW[I,J,NR] := TRPROD( TRMATU[I,ST] , TR_DUW[J] , NR );

               FOR K := 0 TO I DO BEGIN

                   (*  TAYLORKOEFFIZIENTEN VON  U[I] * (U[J]*W[J])' * U[K]  *)
                   (*  UND DEM INTEGRAL DAVON BERECHNEN                    *)
           
                   FOR NR := 0 TO P - 1 DO BEGIN
                       TR_UUWU[I,J,K,NR] :=
                              TRPROD( TR_UUW[I,J] , TRMATU[K,ST] , NR );
                       TR_UUWU_IGRAL[I,J,K,NR+1] := TR_UUWU[I,J,K,NR]/(NR+1);
                   END;

                   (* INTEGRATIONSSCHRITT DURCHFHREN  *)

                   TR_UUWU_IGRAL[I,J,K,0] := UUWU_IGRAL[I,J,K];
                   UUWU_IGRAL[I,J,K] :=
                                  PUNKTAUSWERTUNG( H , TR_UUWU_IGRAL[I,J,K] );
                                  
               END; (* FOR , K *)

           END; (* FOR , I *)

       END; (* FOR , J *)

       WRITE( 'RITZ-ST :  ............................................' );
       WRITELN( '...................' , ST : 4 );

   END; (* FOR , ST *)

   (*  RITZ-TENSOR NORMIEREN  *)
    
   RITZ_MAT := UUWU_IGRAL;

   FOR J := 0 TO N DO
       FOR I := 0 TO N DO
           FOR K := 0 TO I DO BEGIN
               RITZ_MAT[I,J,K] := RITZ_MAT[I,J,K] / SQRT( VR_U2[I]*VR_U2[K] );
               RITZ_MAT[K,J,I] := RITZ_MAT[I,J,K];
           END;

END;

(*-------------------------------------------------------------------------*)

END.

(*-------------------------------------------------------------------------*)
(*                                                                         *)
(*  PASCAL-XSC - MODUL  IEP_RAWA                                           *)
(*                                                                         *)
(*  REELLER ANFANGSWERTPROBLEMLSER FR DIE STURM-LIOUVILLE-GLEICHUNG      *)
(*                                                                         *)
(*-------------------------------------------------------------------------*)
