(*-------------------------------------------------------------------------*)
(*                                                                         *)
(*  PASCAL-XSC - MODUL  IEP_TAYK                                 16.03.93  *)
(*                                                                         *)
(*  (c) Markus Neher    Institute for Applied Mathematics                  *)
(*                      D-76128 Karlsruhe University, Germany              *)
(*                      e-mail: ae16@rz.uni-karlsruhe.de                   *)
(*                                                                         *)
(*  OPERATOREN UND SERVICE-ROUTINEN ZUR TAYLOR-ARITHMETIK                  *)
(*                                                                         *)
(*-------------------------------------------------------------------------*)

MODULE iep_tayk;
                                                               
USE GLOBAL i_ari;
 
GLOBAL TYPE RTKO  = GLOBAL DYNAMIC ARRAY[*] OF REAL;
            RTVEC = GLOBAL DYNAMIC ARRAY[*] OF RTKO;
            RTMAT = GLOBAL DYNAMIC ARRAY[*] OF RTVEC;
            ITKO  = GLOBAL DYNAMIC ARRAY[*] OF INTERVAL;
            ITVEC = GLOBAL DYNAMIC ARRAY[*] OF ITKO;
            ITMAT = GLOBAL DYNAMIC ARRAY[*] OF ITVEC;

(*-------------------------------------------------------------------------*)
(*                                                                         *)
(*  Teil I :  Reelle Taylor-Arithmetik                                     *)
(*                                                                         *)
(*-------------------------------------------------------------------------*)

GLOBAL OPERATOR - ( TRM : RTKO ) RES : RTKO[ 0 .. UB( TRM ) ];
  
VAR I   : INTEGER;
    TRO : RTKO[ 0 .. UB( TRM ) ];
  
BEGIN
   FOR I := 0 TO UB( TRM ) DO
       TRO[I] := - TRM[I];
   RES := TRO;
END;
  
(*-------------------------------------------------------------------------*)

GLOBAL OPERATOR + ( TRM , TRO : RTKO ) RES : RTKO[ 0 .. UB( TRM ) ];
  
VAR I   : INTEGER;
    TRQ : RTKO[ 0 .. UB( TRM ) ];
  
BEGIN
   FOR I := 0 TO UB( TRM ) DO
       TRQ[I] := TRM[I] + TRO[I];
   RES := TRQ;
END;
  
(*-------------------------------------------------------------------------*)

GLOBAL OPERATOR + ( X : REAL; TRM : RTKO )
       RES : RTKO[ 0 .. UB( TRM ) ];
  
BEGIN TRM[0] := TRM[0] + X;
   RES := TRM;
END;
  
(*-------------------------------------------------------------------------*)

GLOBAL OPERATOR + ( TRM : RTKO; X : REAL )
       RES : RTKO[ 0 .. UB( TRM ) ];
  
BEGIN TRM[0] := TRM[0] + X;
   RES := TRM;
END;
   
(*-------------------------------------------------------------------------*)

GLOBAL OPERATOR - ( TRM , TRO : RTKO ) RES : RTKO[ 0 .. UB( TRM ) ];
  
VAR I   : INTEGER;
    TRQ : RTKO[ 0 .. UB( TRM ) ];
  
BEGIN
   FOR I := 0 TO UB( TRM ) DO
       TRQ[I] := TRM[I] - TRO[I];
   RES := TRQ;
END;
  
(*-------------------------------------------------------------------------*)

GLOBAL OPERATOR - ( X : REAL; TRM : RTKO )
       RES : RTKO[ 0 .. UB( TRM ) ];
  
VAR I   : INTEGER;
    TRO : RTKO[ 0 .. UB( TRM ) ];
  
BEGIN TRO[0] := X - TRM[0];
   FOR I := 1 TO UB( TRM ) DO
       TRO[I] := - TRM[I];
   RES := TRO;
END;
   
(*--------------------------------------------------------------------------*)

GLOBAL OPERATOR - ( TRM : RTKO; X : REAL )
       RES : RTKO[ 0 .. UB( TRM ) ];
  
BEGIN TRM[0] := TRM[0] - X;
   RES := TRM;
END;
   
(*-------------------------------------------------------------------------*)

GLOBAL OPERATOR * ( X : REAL; TRM : RTKO )
       RES : RTKO[ 0 .. UB( TRM ) ];
  
VAR I   : INTEGER;
    TRO : RTKO[ 0 .. UB( TRM ) ];
  
BEGIN
   FOR I := 0 TO UB( TRM ) DO
       TRO[I] := X * TRM[I];
   RES := TRO;
END;
   
(*-------------------------------------------------------------------------*)

GLOBAL OPERATOR * ( TRM : RTKO; X : REAL )
       RES : RTKO[ 0 .. UB( TRM ) ];
  
VAR I   : INTEGER;
    TRO : RTKO[ 0 .. UB( TRM ) ];
  
BEGIN
   FOR I := 0 TO UB( TRM ) DO
       TRO[I] := TRM[I] * X;
   RES := TRO;
END;
   
(*-------------------------------------------------------------------------*)

GLOBAL OPERATOR / ( TRM : RTKO; X : REAL )
       RES : RTKO[ 0 .. UB( TRM ) ];
  
VAR I   : INTEGER;
    TRO : RTKO[ 0 .. UB( TRM ) ];
  
BEGIN
   FOR I := 0 TO UB( TRM ) DO 
       TRO[I] := TRM[I] / X;
   RES := TRO;
END;
   
(*-------------------------------------------------------------------------*)

GLOBAL FUNCTION TRPROD( TRM , TRO : RTKO; NR : INTEGER ) : REAL;
  
VAR I  : INTEGER; 
    X : REAL;

BEGIN X := 0;
   FOR I := 0 TO NR DO
       X := X + TRM[I] * TRO[NR-I];
   TRPROD := X;
END;
  
(*-------------------------------------------------------------------------*)

GLOBAL FUNCTION TKO( P : INTEGER; X : REAL ) : RTKO[0..P];
  
VAR I     : INTEGER;
    TRM   : RTKO[0..P];
  
BEGIN 
   TRM[0] := X;
   FOR I := 1 TO P DO
       TRM[I] := 0;
   TKO := TRM;
END;
  
(*-------------------------------------------------------------------------*)

GLOBAL FUNCTION TXKO( P : INTEGER; X : REAL ) : RTKO[0..P];
 
VAR I     : INTEGER;
    TRM   : RTKO[0..P];
  
BEGIN 
   TRM[0] := X;
   TRM[1] := 1;
   FOR I := 2 TO P DO
       TRM[I] := 0;
    TXKO := TRM;
END;
     
(*-------------------------------------------------------------------------*)

GLOBAL FUNCTION COSKO( P : INTEGER; X , A : REAL ) : RTKO[0..P];
 
(*  TAYLORKOEFFIZIENTEN VON COS( A * X )  *)

VAR I   : INTEGER;
    TRM : RTKO[0..P];

BEGIN TRM[0] := COS( A * X ); 
   TRM[1] := - A * SIN( A * X );

   FOR I := 2 TO P DO
       TRM[I] := - SQR( A ) * TRM[I-2] / ( ( I - 1 ) * I );
        
   COSKO := TRM;
    
END;

(*-------------------------------------------------------------------------*)

GLOBAL FUNCTION SINKO( P : INTEGER; X , A : REAL ) : RTKO[0..P];
 
(*  TAYLORKOEFFIZIENTEN VON SIN( A * X )  *)

VAR I   : INTEGER;
    TRM : RTKO[0..P];

BEGIN TRM[0] := SIN( A * X ); 
   TRM[1] := A * COS( A * X );

   FOR I := 2 TO P DO
       TRM[I] := - SQR( A ) * TRM[I-2] / ( ( I - 1 ) * I );
        
   SINKO := TRM;
    
END;

(*-------------------------------------------------------------------------*)

GLOBAL FUNCTION COSHKO( P : INTEGER; X , A : REAL ) : RTKO[0..P];
 
(*  TAYLORKOEFFIZIENTEN VON COSH( A * X )  *)

VAR I   : INTEGER;
    TRM : RTKO[0..P];

BEGIN TRM[0] := COSH( A * X ); 
   TRM[1] := A * SINH( A * X );

   FOR I := 2 TO P DO
       TRM[I] := SQR( A ) * TRM[I-2] / ( ( I - 1 ) * I );
        
   COSHKO := TRM;
    
END;

(*-------------------------------------------------------------------------*)

GLOBAL FUNCTION TRMATRD( P : INTEGER ) : RTMAT[1..2,1..2,0..P];
  
VAR TRZERO , TRONE : RTKO[0..P];
    TRMA           : RTMAT[1..2,1..2,0..P];
  
BEGIN TRZERO := TKO( P , 0 );
   TRONE := TKO( P , 1 );

   TRMA[1,1] := TRONE;
   TRMA[1,2] := TRZERO;
   TRMA[2,1] := TRZERO;
   TRMA[2,2] := TRONE;

   TRMATRD := TRMA;
END;
  
(*-------------------------------------------------------------------------*)

GLOBAL PROCEDURE TRMATPROD( TRMA, TRMB: RTMAT; VAR TRMC: RTMAT; NR: INTEGER );
 
VAR I , J , K : INTEGER;
    Z         : ARRAY[1..2,1..2,1..2] OF REAL;
  
BEGIN 
   FOR I := 1 TO 2 DO
       FOR J := 1 TO 2 DO BEGIN
           TRMC[I,J,NR] := 0;
           FOR K := 1 TO 2 DO BEGIN
               Z[K,I,J] := TRPROD( TRMA[I,K] , TRMB[K,J] , NR );
               TRMC[I,J,NR] := TRMC[I,J,NR] + Z[K,I,J]; 
           END;
       END;
END;

(*-------------------------------------------------------------------------*)
(*                                                                         *)
(*  Teil II :  Intervall - Taylor-Arithmetik                               *)
(*                                                                         *)
(*-------------------------------------------------------------------------*)

GLOBAL OPERATOR - ( TIM : ITKO ) RES : ITKO[ 0 .. UB( TIM ) ];
  
VAR I   : INTEGER;
    TIO : ITKO[ 0 .. UB( TIM ) ];
  
BEGIN
   FOR I := 0 TO UB( TIM ) DO
       TIO[I] := - TIM[I];
   RES := TIO;
END;
  
(*-------------------------------------------------------------------------*)

GLOBAL OPERATOR + ( TIM , TIO : ITKO ) RES : ITKO[ 0 .. UB( TIM ) ];
  
VAR I   : INTEGER;
    TIQ : ITKO[ 0 .. UB( TIM ) ];
  
BEGIN
   FOR I := 0 TO UB( TIM ) DO
       TIQ[I] := TIM[I] + TIO[I];
   RES := TIQ;
END;
  
(*-------------------------------------------------------------------------*)

GLOBAL OPERATOR + ( IX : INTERVAL; TIM : ITKO )
       RES : ITKO[ 0 .. UB( TIM ) ];
  
BEGIN TIM[0] := TIM[0] + IX;
   RES := TIM;
END;
  
(*-------------------------------------------------------------------------*)

GLOBAL OPERATOR + ( TIM : ITKO; IX : INTERVAL )
       RES : ITKO[ 0 .. UB( TIM ) ];
  
BEGIN TIM[0] := TIM[0] + IX;
   RES := TIM;
END;
   
(*-------------------------------------------------------------------------*)

GLOBAL OPERATOR - ( TIM , TIO : ITKO ) RES : ITKO[ 0 .. UB( TIM ) ];
  
VAR I   : INTEGER;
    TIQ : ITKO[ 0 .. UB( TIM ) ];
  
BEGIN
   FOR I := 0 TO UB( TIM ) DO
       TIQ[I] := TIM[I] - TIO[I];
   RES := TIQ;
END;
  
(*-------------------------------------------------------------------------*)

GLOBAL OPERATOR - ( IX : INTERVAL; TIM : ITKO )
       RES : ITKO[ 0 .. UB( TIM ) ];
  
VAR I   : INTEGER;
    TIO : ITKO[ 0 .. UB( TIM ) ];
  
BEGIN TIO[0] := IX - TIM[0];
   FOR I := 1 TO UB( TIM ) DO
       TIO[I] := - TIM[I];
   RES := TIO;
END;
   
(*--------------------------------------------------------------------------*)

GLOBAL OPERATOR - ( TIM : ITKO; IX : INTERVAL )
       RES : ITKO[ 0 .. UB( TIM ) ];
  
BEGIN TIM[0] := TIM[0] - IX;
   RES := TIM;
END;
   
(*-------------------------------------------------------------------------*)

GLOBAL OPERATOR * ( J : INTEGER; TIM : ITKO )
       RES : ITKO[ 0 .. UB( TIM ) ];
  
VAR I   : INTEGER;
    TIO : ITKO[ 0 .. UB( TIM ) ];
  
BEGIN
   FOR I := 0 TO UB( TIM ) DO
       TIO[I] := J * TIM[I];
   RES := TIO;
END;
   
(*-------------------------------------------------------------------------*)

GLOBAL OPERATOR * ( TIM : ITKO; J : INTEGER )
       RES : ITKO[ 0 .. UB( TIM ) ];
  
VAR I   : INTEGER;
    TIO : ITKO[ 0 .. UB( TIM ) ];
  
BEGIN
   FOR I := 0 TO UB( TIM ) DO
       TIO[I] := TIM[I] * J;
   RES := TIO;
END;
   
(*-------------------------------------------------------------------------*)

GLOBAL OPERATOR * ( IX : INTERVAL; TIM : ITKO )
       RES : ITKO[ 0 .. UB( TIM ) ];
  
VAR I   : INTEGER;
    TIO : ITKO[ 0 .. UB( TIM ) ];
  
BEGIN
   FOR I := 0 TO UB( TIM ) DO
       TIO[I] := IX * TIM[I];
   RES := TIO;
END;
   
(*-------------------------------------------------------------------------*)

GLOBAL OPERATOR * ( TIM : ITKO; IX : INTERVAL )
       RES : ITKO[ 0 .. UB( TIM ) ];
  
VAR I   : INTEGER;
    TIO : ITKO[ 0 .. UB( TIM ) ];
  
BEGIN
   FOR I := 0 TO UB( TIM ) DO
       TIO[I] := TIM[I] * IX;
   RES := TIO;
END;
   
(*-------------------------------------------------------------------------*)

GLOBAL OPERATOR / ( TIM : ITKO; J : INTEGER )
       RES : ITKO[ 0 .. UB( TIM ) ];
  
VAR I   : INTEGER;
    TIO : ITKO[ 0 .. UB( TIM ) ];
  
BEGIN
   FOR I := 0 TO UB( TIM ) DO 
       TIO[I] := TIM[I] / J;
   RES := TIO;
END;
   
(*-------------------------------------------------------------------------*)

GLOBAL OPERATOR / ( TIM : ITKO; IX : INTERVAL )
       RES : ITKO[ 0 .. UB( TIM ) ];
  
VAR I   : INTEGER;
    TIO : ITKO[ 0 .. UB( TIM ) ];
  
BEGIN
   FOR I := 0 TO UB( TIM ) DO 
       TIO[I] := TIM[I] / IX;
   RES := TIO;
END;
   
(*-------------------------------------------------------------------------*)

GLOBAL FUNCTION TIPROD( TIM , TIO : ITKO; NR : INTEGER ) : INTERVAL;
  
VAR I  : INTEGER; 
    IX : INTERVAL;

BEGIN IX := 0;
   FOR I := 0 TO NR DO
       IX := IX + TIM[I] * TIO[NR-I];
   TIPROD := IX;
END;
  
(*-------------------------------------------------------------------------*)

GLOBAL FUNCTION TKO( P : INTEGER; IX : INTERVAL ) : ITKO[0..P];
  
VAR I     : INTEGER;
    TIM   : ITKO[0..P];
  
BEGIN 
   TIM[0] := IX;
   FOR I := 1 TO P DO
       TIM[I] := 0;
   TKO := TIM;
END;
  
(*-------------------------------------------------------------------------*)

GLOBAL FUNCTION TXKO( P : INTEGER; IX : INTERVAL ) : ITKO[0..P];
 
VAR I     : INTEGER;
    TIM   : ITKO[0..P];
  
BEGIN 
   TIM[0] := IX;
   TIM[1] := 1;
   FOR I := 2 TO P DO
       TIM[I] := 0;
    TXKO := TIM;
END;
     
(*-------------------------------------------------------------------------*)

GLOBAL FUNCTION COSKO( P : INTEGER; IX , IA : INTERVAL ) : ITKO[0..P];
 
(*  TAYLORKOEFFIZIENTEN VON COS( A * X )  *)

VAR I   : INTEGER;
    TIM : ITKO[0..P];

BEGIN TIM[0] := COS( IA * IX ); 
   TIM[1] := - IA * SIN( IA * IX );

   FOR I := 2 TO P DO
       TIM[I] := - SQR( IA ) * TIM[I-2] / ( ( I - 1 ) * I );
        
   COSKO := TIM;
    
END;

(*-------------------------------------------------------------------------*)

GLOBAL FUNCTION SINKO( P : INTEGER; IX , IA : INTERVAL ) : ITKO[0..P];
 
(*  TAYLORKOEFFIZIENTEN VON SIN( A * X )  *)

VAR I   : INTEGER;
    TIM : ITKO[0..P];

BEGIN TIM[0] := SIN( IA * IX ); 
   TIM[1] := IA * COS( IA * IX );

   FOR I := 2 TO P DO
       TIM[I] := - SQR( IA ) * TIM[I-2] / ( ( I - 1 ) * I );
        
   SINKO := TIM;
    
END;

(*-------------------------------------------------------------------------*)

GLOBAL FUNCTION COSHKO( P : INTEGER; IX , IA : INTERVAL ) : ITKO[0..P];
 
(*  TAYLORKOEFFIZIENTEN VON COSH( A * X )  *)

VAR I   : INTEGER;
    TIM : ITKO[0..P];

BEGIN TIM[0] := COSH( IA * IX ); 
   TIM[1] := IA * SINH( IA * IX );

   FOR I := 2 TO P DO
       TIM[I] := SQR( IA ) * TIM[I-2] / ( ( I - 1 ) * I );
        
   COSHKO := TIM;
    
END;

(*-------------------------------------------------------------------------*)

GLOBAL FUNCTION TIMATID( P : INTEGER ) : ITMAT[1..2,1..2,0..P];
  
VAR TIZERO , TIONE : ITKO[0..P];
    TIMA           : ITMAT[1..2,1..2,0..P];
  
BEGIN TIZERO := TKO( P , INTVAL( 0 ) );
   TIONE := TKO( P , INTVAL( 1 ) );

   TIMA[1,1] := TIONE;
   TIMA[1,2] := TIZERO;
   TIMA[2,1] := TIZERO;
   TIMA[2,2] := TIONE;

   TIMATID := TIMA;
END;
  
(*-------------------------------------------------------------------------*)

GLOBAL PROCEDURE TIMATPROD( TIMA, TIMB: ITMAT; VAR TIMC: ITMAT; NR: INTEGER );
 
VAR I , J , K : INTEGER;
    Z         : ARRAY[1..2,1..2,1..2] OF INTERVAL;
  
BEGIN 
   FOR I := 1 TO 2 DO
       FOR J := 1 TO 2 DO BEGIN
           TIMC[I,J,NR] := 0;
           FOR K := 1 TO 2 DO BEGIN
               Z[K,I,J] := TIPROD( TIMA[I,K] , TIMB[K,J] , NR );
               TIMC[I,J,NR] := TIMC[I,J,NR] + Z[K,I,J]; 
           END;
       END;
END;

(*-------------------------------------------------------------------------*)
(*                                                                         *)
(*  Teil III :  Transfer - Funktionen                                      *)
(*                                                                         *)
(*-------------------------------------------------------------------------*)

GLOBAL FUNCTION MID( TIM : ITKO ) : RTKO[ LB( TIM ) .. UB( TIM ) ];

VAR I   : INTEGER;
    TRM : RTKO[ LB( TIM ) .. UB( TIM ) ];
     
BEGIN
   FOR I := LB( TIM ) TO UB( TIM ) DO
       TRM[I] := MID( TIM[I] );
   MID := TRM;
END;  

(*-------------------------------------------------------------------------*)

GLOBAL FUNCTION MID( TIVECU : ITVEC ) :
       RTVEC[ LB(TIVECU) .. UB(TIVECU) , LB(TIVECU,2) .. UB(TIVECU,2) ];

VAR I      : INTEGER;
    TRVECU : RTVEC[ LB(TIVECU) .. UB(TIVECU) , LB(TIVECU,2) .. UB(TIVECU,2) ];
     
BEGIN
   FOR I := LB( TIVECU ) TO UB( TIVECU ) DO
       TRVECU[I] := MID( TIVECU[I] );
   MID := TRVECU;
END;  

(*-------------------------------------------------------------------------*)

GLOBAL FUNCTION MID( TIMA : ITMAT ) :
  RTMAT[ LB(TIMA)..UB(TIMA), LB(TIMA,2)..UB(TIMA,2), LB(TIMA,3)..UB(TIMA,3) ];

VAR I  : INTEGER;
  TRMA : 
  RTMAT[ LB(TIMA)..UB(TIMA), LB(TIMA,2)..UB(TIMA,2), LB(TIMA,3)..UB(TIMA,3) ];
     
BEGIN
   FOR I := LB( TIMA ) TO UB( TIMA ) DO
       TRMA[I] := MID( TIMA[I] );
   MID := TRMA;
END;  

(*-------------------------------------------------------------------------*)

END.

(*-------------------------------------------------------------------------*)
(*                                                                         *)
(*  PASCAL-XSC - MODUL  IEP_TAYK                                           *)
(*                                                                         *)
(*  OPERATOREN UND SERVICE-ROUTINEN ZUR TAYLOR-ARITHMETIK                  *)
(*                                                                         *)
(*-------------------------------------------------------------------------*)
