(*-------------------------------------------------------------------------*)
(*                                                                         *)
(*  PASCAL-XSC - MODUL  SPLINES                                  03.05.93  *)
(*                                                                         *)
(*  (c) Markus Neher    Institute for Applied Mathematics                  *)
(*                      D-76128 Karlsruhe University, Germany              *)
(*                      e-mail: ae16@rz.uni-karlsruhe.de                   *)
(*                                                                         *)
(*  KONSTRUIERT AUS DEN FUNKTIONSWERTEN EINER SYMMETRISCHEN FUNKTION EINEN *)
(*  SYMMETRISCHEN, KUBISCHEN, ZWEIMAL STETIG DIFFERENZIERBAREN SPLINE      *)
(*                                                                         *)
(*  BERECHNET WERDEN DIE WERTE DER ERSTEN, ZWEITEN UND DRITTEN ABLEITUNG   *)
(*  DER FUNKTION IN DEN STTZSTELLEN                                       *)
(*                                                                         *)
(*  NORMIERUNGEN:                                                          *)
(*                                                                         *)
(*    DIE FUNKTION IST DEFINIERT AUF DEM INTERVALL [ 0 , UPBD ] ,          *)
(*    SYMMETRISCH ZUR INTERVALLMITTE UND ES LIEGEN ( N + 1 ) GLEICH-       *)
(*    ABSTNDIGE STTZSTELLEN VOR                                          *)
(*                                                                         *)
(*-------------------------------------------------------------------------*)

MODULE splines;

USE i_ari , mv_ari , mvi_ari , lss , ilss , service; 

GLOBAL TYPE RSPLINE  = GLOBAL DYNAMIC ARRAY[*,*] OF REAL;
(*  ERSTE KOMPONENTE  : I-TE ABLEITUNG    *)
(*  ZWEITE KOMPONENTE : J-TE STTZSTELLE  *)
GLOBAL TYPE ISPLINE  = GLOBAL DYNAMIC ARRAY[*,*] OF INTERVAL;
GLOBAL TYPE RSVECTOR = GLOBAL DYNAMIC ARRAY[ * ] OF RSPLINE;
GLOBAL TYPE ISVECTOR = GLOBAL DYNAMIC ARRAY[ * ] OF ISPLINE;

(*--------------------------------------------------------------------------*)
 
GLOBAL PROCEDURE SPLINE( UPBD : REAL;  VAR P : RSPLINE );
 
(*  BERECHNET EINEN KUBISCHEN SYMMETRISCHEN REELLEN SPLINE  *)
(*  IM INTERVALL  [ 0 , UPBD ]                              *)

VAR I , J , M , N , IER : INTEGER;
    H , H2              : REAL;
    VRB                 : RVECTOR[ 1..UB( P , 2 ) DIV 2 ];
    VIC                 : IVECTOR[ 1..UB( P , 2 ) DIV 2 ];
    MRA                 : RMATRIX[ 1..UB( P , 2 ) DIV 2 , 1..UB(P,2) DIV 2 ];

BEGIN M := UB( P , 2 );
   N := M DIV 2;
   H := UPBD / M;
   H2 := SQR( H );

   MRA := NULL( MRA );
   VIC := NULL( VIC );

   (*  KUBISCHE EXTRAPOLATION DER ZWEITEN ABLEITUNG AM RAND  *)
   (*  P[2,0] := ( 2 * P[0,0] - 5 * P[0,1] + 4 * P[0,2] - P[0,3] ) / H2;  *)

   (*  QUADRATISCHE EXTRAPOLATION DER ZWEITEN ABLEITUNG AM RAND  *)
   (*  P[2,0] := ( P[0,0] - 2 * P[0,1] + P[0,2] ) / H2;  *)
   
   (*  GEMISCHTE EXTRAPOLATION DER ZWEITEN ABLEITUNG AM RAND ( 5 : 2 )  *)
   P[2,0] := ( 12*P[0,0] - 29*P[0,1] + 22*P[0,2] - 5*P[0,3] ) / ( 7*H2 );
   
   FOR I := 1 TO N - 1 DO BEGIN
       MRA[I,I] := 4;         
       MRA[I,I+1] := 1;
       MRA[I+1,I] := 1;
       VRB[I] := 6 * ( P[0,I+1] - 2 * P[0,I] + P[0,I-1] ) / H2;
   END; 
   IF ODD ( M ) THEN BEGIN
      MRA[N,N] := 5;
      VRB[N] := 6 * ( P[0,N-1] - P[0,N] ) / H2;
      END
   ELSE BEGIN
      MRA[N,N] := 4;
      MRA[N,N-1] := 2;
      VRB[N] := 12 * ( P[0,N-1] - P[0,N] ) / H2;
   END; 

   VRB[1] := VRB[1] - P[2,0];
    
   (*  LGS DER ZWEITEN ABLEITUNGEN DES SPLINES IN DEN STTZSTELLEN  *)
   
   LSS( MRA , VRB , VIC , IER );

   FOR I := 1 TO N DO BEGIN
       P[2,I] := MID( VIC[I] );
       P[1,I-1] := ( P[0,I] - P[0,I-1] ) / H - H * ( P[2,I]/2 + P[2,I-1] )/3;
       P[3,I-1] := ( P[2,I] - P[2,I-1] ) / H;
   END;

   (*  JETZT WERTE AUS DER LINKEN INTERVALLHLFTE IN DIE RECHTE FORTSETZEN  *)
    
   IF ODD ( M ) THEN BEGIN
      P[1,N] := - H * P[2,N] / 2;
      P[3,N] := 0;
      FOR I := N DOWNTO 0 DO BEGIN
          P[0,M-I] :=   P[0,I];
          P[1,M-I] := - P[1,I];
          P[2,M-I] :=   P[2,I];
      END;
      END
   ELSE BEGIN
      P[1,N] := 0;
      FOR I := N-1 DOWNTO 0 DO BEGIN
          P[0,M-I] :=   P[0,I];
          P[1,M-I] := - P[1,I];
          P[2,M-I] :=   P[2,I];
      END;
   END; 
    
   (*  DRITTE ABLEITUNG IST NICHT SYMMETRISCH  !!! *)

   FOR I := N DOWNTO 1 DO 
       P[3,M-I] := - P[3,I-1];
       
   (*  DRITTE ABLEITUNG AM RECHTEN RAND NICHT GEBRAUCHT  *)

   P[3,M] := 0;
       
END;

(*--------------------------------------------------------------------------*)

GLOBAL PROCEDURE SPLINE( UPBD : INTERVAL;  VAR P : ISPLINE );
 
(*  BERECHNET EINEN KUBISCHEN SYMMETRISCHEN INTERVALLSPLINE  *)

VAR I , J , M , N , IER : INTEGER;
    H , H2              : INTERVAL;
    VIB , VIC           : IVECTOR[ 1..UB( P , 2 ) DIV 2 ];
    MIA                 : IMATRIX[ 1..UB( P , 2 ) DIV 2 , 1..UB(P,2) DIV 2 ];

BEGIN M := UB( P , 2 );
   N := M DIV 2;
   H := UPBD / M;
   H2 := SQR( H );

   MIA := NULL( MIA );
   VIC := NULL( VIC );

   (*  KUBISCHE EXTRAPOLATION DER ZWEITEN ABLEITUNG AM RAND  *)
   (*  P[2,0] := ( 2 * P[0,0] - 5 * P[0,1] + 4 * P[0,2] - P[0,3] ) / H2;  *)

   (*  QUADRATISCHE EXTRAPOLATION DER ZWEITEN ABLEITUNG AM RAND  *)
   (*  P[2,0] := ( P[0,0] - 2 * P[0,1] + P[0,2] ) / H2;  *)
   
   (*  GEMISCHTE EXTRAPOLATION DER ZWEITEN ABLEITUNG AM RAND ( 5 : 2 )  *)
   P[2,0] := ##( 12*P[0,0] - 29*P[0,1] + 22*P[0,2] - 5 * P[0,3] ) / ( 7*H2 );
   
   FOR I := 1 TO N - 1 DO BEGIN
       MIA[I,I] := 4;         
       MIA[I,I+1] := 1;
       MIA[I+1,I] := 1;
       VIB[I] := ##( 6 * P[0,I+1] - 12 * P[0,I] + 6 * P[0,I-1] ) / H2;
   END; 
   IF ODD ( M ) THEN BEGIN
      MIA[N,N] := 5;
      VIB[N] := ##( 6 * P[0,N-1] - 6 * P[0,N] ) / H2;
      END
   ELSE BEGIN
      MIA[N,N] := 4;
      MIA[N,N-1] := 2;
      VIB[N] := ##( 12 * P[0,N-1] - 12 * P[0,N] ) / H2;
   END; 

   VIB[1] := VIB[1] - P[2,0];
    
   (*  LGS DER ZWEITEN ABLEITUNGEN DES SPLINES IN DEN STTZSTELLEN  *)
   
   LSS( MIA , VIB , VIC , IER );

   FOR I := 1 TO N DO BEGIN
       P[2,I] := VIC[I];
       P[1,I-1] := ( P[0,I] - P[0,I-1] ) / H - H * ( P[2,I]/2 + P[2,I-1] )/3;
       P[3,I-1] := ( P[2,I] - P[2,I-1] ) / H;
   END;

   (*  JETZT WERTE AUS DER LINKEN INTERVALLHLFTE IN DIE RECHTE FORTSETZEN  *)
    
   IF ODD ( M ) THEN BEGIN
      P[1,N] := - H * P[2,N] / 2;
      P[3,N] := 0;
      FOR I := N DOWNTO 0 DO BEGIN
          P[0,M-I] :=   P[0,I];
          P[1,M-I] := - P[1,I];
          P[2,M-I] :=   P[2,I];
      END;
      END
   ELSE BEGIN
      P[1,N] := 0;
      FOR I := N-1 DOWNTO 0 DO BEGIN
          P[0,M-I] :=   P[0,I];
          P[1,M-I] := - P[1,I];
          P[2,M-I] :=   P[2,I];
      END;
   END; 
    
   (*  DRITTE ABLEITUNG IST NICHT SYMMETRISCH  !!! *)

   FOR I := N DOWNTO 1 DO 
       P[3,M-I] := - P[3,I-1];
       
   (*  DRITTE ABLEITUNG AM RECHTEN RAND NICHT GEBRAUCHT  *)

   P[3,M] := 0;
       
END;

(*--------------------------------------------------------------------------*)

GLOBAL PROCEDURE SPLINE( VAR P : RSPLINE );
 
(*  BERECHNET EINEN KUBISCHEN SYMMETRISCHEN SPLINE  *)

(*  IN DIESER ROUTINE WIRD BEI DER k - TEN ABLEITUNG DER     *)
(*  h^k - FACHE WERT BERECHNET , h WIRD IN DIESEM            *)
(*  UNTERPROGRAMM NICHT GEBRAUCHT !!!                        *)
(*  (REDUZIERT ANZAHL DER RUNDUNGEN)                         *)

VAR I , J , M , N , IER : INTEGER;
    VRB                 : RVECTOR[ 1..UB( P , 2 ) DIV 2 ];
    VIC                 : IVECTOR[ 1..UB( P , 2 ) DIV 2 ];
    MRA                 : RMATRIX[ 1..UB( P , 2 ) DIV 2 , 1..UB(P,2) DIV 2 ];

BEGIN M := UB( P , 2 );
   N := M DIV 2;

   MRA := NULL( MRA );
   VIC := NULL( VIC );

   (*  KUBISCHE EXTRAPOLATION DER ZWEITEN ABLEITUNG AM RAND  *)
   (*  P[2,0] := ( 2 * P[0,0] - 5 * P[0,1] + 4 * P[0,2] - P[0,3] );  *)

   (*  QUADRATISCHE EXTRAPOLATION DER ZWEITEN ABLEITUNG AM RAND  *)
   (*  P[2,0] := ( P[0,0] - 2 * P[0,1] + P[0,2] );  *)
   
   (*  GEMISCHTE EXTRAPOLATION DER ZWEITEN ABLEITUNG AM RAND ( 5 : 2 )  *)
   P[2,0] := #*( 12 * P[0,0] - 29 * P[0,1] + 22 * P[0,2] - 5 * P[0,3] ) / 7;
   
   FOR I := 1 TO N - 1 DO BEGIN
       MRA[I,I] := 4;         
       MRA[I,I+1] := 1;
       MRA[I+1,I] := 1;
       VRB[I] := #*( 6 * P[0,I+1] - 12 * P[0,I] + 6 * P[0,I-1] );
   END; 
   IF ODD ( M ) THEN BEGIN
      MRA[N,N] := 5;
      VRB[N] := #*( 6 * P[0,N-1] - 6 * P[0,N] );
      END
   ELSE BEGIN
      MRA[N,N] := 4;
      MRA[N,N-1] := 2;
      VRB[N] := #*( 12 * P[0,N-1] - 12 * P[0,N] );
   END; 

   VRB[1] := #*( 6 * P[0,2] - 12 * P[0,1] + 6 * P[0,0] - P[2,0] );
    
   (*  LGS DER ZWEITEN ABLEITUNGEN DES SPLINES IN DEN ST"UTZSTELLEN  *)
   
   LSS( MRA , VRB , VIC , IER );

   FOR I := 1 TO N DO BEGIN
       P[2,I] := MID( VIC[I] );
       P[3,I-1] := ( P[2,I] - P[2,I-1] );
       P[1,I-1] := #*( 6 * P[0,I] - 6 * P[0,I-1] - P[2,I] - 2 * P[2,I-1] ) / 6;
   END;

   (*  JETZT WERTE AUS DER LINKEN INTERVALLH"ALFTE IN DIE RECHTE FORTSETZEN  *)
    
   IF ODD ( M ) THEN BEGIN
      P[1,N] := - P[2,N] / 2;
      P[3,N] := 0;
      FOR I := N DOWNTO 0 DO BEGIN
          P[0,M-I] :=   P[0,I];
          P[1,M-I] := - P[1,I];
          P[2,M-I] :=   P[2,I];
      END;
      END
   ELSE BEGIN
      P[1,N] := 0;
      FOR I := N-1 DOWNTO 0 DO BEGIN
          P[0,M-I] :=   P[0,I];
          P[1,M-I] := - P[1,I];
          P[2,M-I] :=   P[2,I];
      END;
   END; 
    
   (*  DRITTE ABLEITUNG IST NICHT SYMMETRISCH  !!! *)

   FOR I := N DOWNTO 1 DO 
       P[3,M-I] := - P[3,I-1];
       
   (*  DRITTE ABLEITUNG AM RECHTEN RAND NICHT GEBRAUCHT  *)

   P[3,M] := 0;
       
END;

(*--------------------------------------------------------------------------*)

GLOBAL PROCEDURE SPLINE( VAR P : ISPLINE );
 
(*  BERECHNET EINEN KUBISCHEN SYMMETRISCHEN INTERVALLSPLINE  *)

(*  IN DIESER ROUTINE WIRD BEI DER k - TEN ABLEITUNG DER     *)
(*  h^k - FACHE WERT BERECHNET , h WIRD IN DIESEM            *)
(*  UNTERPROGRAMM NICHT GEBRAUCHT !!!                        *)
(*  (REDUZIERT ANZAHL DER RUNDUNGEN)                         *)

VAR I , J , M , N , IER : INTEGER;
    VIB , VIC           : IVECTOR[ 1..UB( P , 2 ) DIV 2 ];
    MIA                 : IMATRIX[ 1..UB( P , 2 ) DIV 2 , 1..UB(P,2) DIV 2 ];

BEGIN M := UB( P , 2 );
   N := M DIV 2;

   MIA := NULL( MIA );
   VIC := NULL( VIC );

   (*  KUBISCHE EXTRAPOLATION DER ZWEITEN ABLEITUNG AM RAND  *)
   (*  P[2,0] := ( 2 * P[0,0] - 5 * P[0,1] + 4 * P[0,2] - P[0,3] );  *)

   (*  QUADRATISCHE EXTRAPOLATION DER ZWEITEN ABLEITUNG AM RAND  *)
   (*  P[2,0] := ( P[0,0] - 2 * P[0,1] + P[0,2] );  *)
   
   (*  GEMISCHTE EXTRAPOLATION DER ZWEITEN ABLEITUNG AM RAND ( 5 : 2 )  *)
   P[2,0] := ##( 12 * P[0,0] - 29 * P[0,1] + 22 * P[0,2] - 5 * P[0,3] ) / 7;
   
   FOR I := 1 TO N - 1 DO BEGIN
       MIA[I,I] := 4;         
       MIA[I,I+1] := 1;
       MIA[I+1,I] := 1;
       VIB[I] := ##( 6 * P[0,I+1] - 12 * P[0,I] + 6 * P[0,I-1] );
   END; 
   IF ODD ( M ) THEN BEGIN
      MIA[N,N] := 5;
      VIB[N] := ##( 6 * P[0,N-1] - 6 * P[0,N] );
      END
   ELSE BEGIN
      MIA[N,N] := 4;
      MIA[N,N-1] := 2;
      VIB[N] := ##( 12 * P[0,N-1] - 12 * P[0,N] );
   END; 

   VIB[1] := ##( 6 * P[0,2] - 12 * P[0,1] + 6 * P[0,0] - P[2,0] );
    
   (*  LGS DER ZWEITEN ABLEITUNGEN DES SPLINES IN DEN ST"UTZSTELLEN  *)
   
   LSS( MIA , VIB , VIC , IER );

   FOR I := 1 TO N DO BEGIN
       P[2,I] := VIC[I];
       P[3,I-1] := ( P[2,I] - P[2,I-1] );
       P[1,I-1] := ##( 6 * P[0,I] - 6 * P[0,I-1] - P[2,I] - 2 * P[2,I-1] ) / 6;
   END;

   (*  JETZT WERTE AUS DER LINKEN INTERVALLH"ALFTE IN DIE RECHTE FORTSETZEN  *)
    
   IF ODD ( M ) THEN BEGIN
      P[1,N] := - P[2,N] / 2;
      P[3,N] := 0;
      FOR I := N DOWNTO 0 DO BEGIN
          P[0,M-I] :=   P[0,I];
          P[1,M-I] := - P[1,I];
          P[2,M-I] :=   P[2,I];
      END;
      END
   ELSE BEGIN
      P[1,N] := 0;
      FOR I := N-1 DOWNTO 0 DO BEGIN
          P[0,M-I] :=   P[0,I];
          P[1,M-I] := - P[1,I];
          P[2,M-I] :=   P[2,I];
      END;
   END; 
    
   (*  DRITTE ABLEITUNG IST NICHT SYMMETRISCH  !!! *)

   FOR I := N DOWNTO 1 DO 
       P[3,M-I] := - P[3,I-1];
       
   (*  DRITTE ABLEITUNG AM RECHTEN RAND NICHT GEBRAUCHT  *)

   P[3,M] := 0;
       
END;

(*--------------------------------------------------------------------------*)

GLOBAL PROCEDURE SWRITE( NAME : STRING; UPBD : REAL; P : RSPLINE );
 
(*  DRUCKT EINEN REELLEN SPLINE  *)

VAR I , J , UB2 : INTEGER;
 
BEGIN UB2 := UB( P , 2 ) DIV 2;

   FOR J := LB( P , 2 ) TO UB2 DO BEGIN

       WRITE( NAME , '  (' , J * UPBD / UB2 : 6 : 4 , ') = ' );
       WRITELN( P[0,J] );

       WRITE( 'D' , NAME , ' (' , J * UPBD / UB2 : 6 : 4 , ') = ' );
       WRITELN( P[1,J] );

       FOR I := 2 TO UB( P , 1 ) DO BEGIN
           WRITE( 'D' , I : 1 , NAME );
           WRITE( '(' , J * UPBD / UB2 : 6 : 4 , ') = ' );
           WRITELN( P[I,J] );
       END;

       WRITELN;

   END;

END;
        
(*--------------------------------------------------------------------------*)

GLOBAL PROCEDURE SWRITE
       ( VAR AUS : TEXT; NAME : STRING; UPBD : REAL; P : RSPLINE );
 
(*  DRUCKT EINEN REELLEN SPLINE  *)

VAR I , J , UB2 : INTEGER;
 
BEGIN UB2 := UB( P , 2 ) DIV 2;

   FOR J := LB( P , 2 ) TO UB2 DO BEGIN

       WRITE( AUS , NAME , '  (' , J * UPBD / UB2 : 6 : 4 , ') = ' );
       WRITELN( AUS , P[0,J] );

       WRITE( AUS , 'D' , NAME , ' (' , J * UPBD / UB2 : 6 : 4 , ') = ' );
       WRITELN( AUS , P[1,J] );

       FOR I := 2 TO UB( P , 1 ) DO BEGIN
           WRITE( AUS , 'D' , I : 1 , NAME );
           WRITE( AUS , '(' , J * UPBD / UB2 : 6 : 4 , ') = ' );
           WRITELN( AUS , P[I,J] );
       END;

       WRITELN( AUS );

   END;

END;
        
(*--------------------------------------------------------------------------*)

GLOBAL PROCEDURE SWRITE( NAME : STRING; UPBD : REAL; P : ISPLINE );
 
(*  DRUCKT EINEN INTERVALLSPLINE  *)

VAR I , J , UB2 : INTEGER;
 
BEGIN UB2 := UB( P , 2 ) DIV 2;

   FOR J := LB( P , 2 ) TO UB2 DO BEGIN

       WRITE( NAME , '  (' , J * UPBD / UB2 : 6 : 4 , ') = ' );
       IWRITELN( P[0,J] );

       WRITE( 'D' , NAME , ' (' , J * UPBD / UB2 : 6 : 4 , ') = ' );
       IWRITELN( P[1,J] );

       FOR I := 2 TO UB( P , 1 ) DO BEGIN
           WRITE( 'D' , I:1 , NAME , '(' , J * UPBD / UB2 : 6 : 4 , ') = ' );
           IWRITELN( P[I,J] );
       END;

       WRITELN;

   END;

END;
        
(*--------------------------------------------------------------------------*)

GLOBAL PROCEDURE SWRITE
       ( VAR AUS : TEXT; NAME : STRING; UPBD : INTERVAL; P : ISPLINE );
 
(*  DRUCKT EINEN INTERVALLSPLINE  *)

VAR I , J , UB2 : INTEGER;
 
BEGIN UB2 := UB( P , 2 ) DIV 2;

   FOR J := LB( P , 2 ) TO UB2 DO BEGIN

       WRITE( AUS , NAME , '  (' , J * UPBD.SUP / UB2 : 6 : 4 , ') = ' );
       IWRITELN( AUS , P[0,J] );

       WRITE( AUS , 'D' , NAME , ' (' , J * UPBD.SUP / UB2 : 6 : 4 , ') = ' );
       IWRITELN( AUS , P[1,J] );

       FOR I := 2 TO UB( P , 1 ) DO BEGIN
           WRITE( AUS , 'D' , I : 1 , NAME );
           WRITE( AUS , '(' , J * UPBD.SUP / UB2 : 6 : 4 , ') = ' );
           IWRITELN( AUS , P[I,J] );
       END;

       WRITELN( AUS );

   END;

END;
        
(*--------------------------------------------------------------------------*)

GLOBAL FUNCTION SFOUKOEFF( UPBD : REAL;  VAR SPP : RSPLINE ) :
       RVECTOR[ 0..UB( SPP , 2 ) ];
 
(*  BERECHNET DIE FOURIER-KOEFFIZIENTEN EINES  *)
(*  KUBISCHEN SYMMETRISCHEN REELLEN SPLINES    *)

VAR I , ST , M : INTEGER;
    FK         : REAL;
    FOUKO      : RVECTOR[ 0..UB( SPP , 2 ) ];
     
BEGIN  M := UB( SPP , 2 );
   FK := 0;
     
   FOR I := 3 DOWNTO 0 DO BEGIN

       FOR ST := 0 TO M - 1 DO
           FK := FK + SPP[I,ST];
    
       FK := FK * UPBD / ( M * ( I + 1 ) );

   END;

   FOUKO[0] := FK / UPBD;
  
   FOR I := 1 TO M DO BEGIN

       FOUKO[I] := 0;

       FOR ST := 0 TO M - 1 DO 
           FOUKO[I] := FOUKO[I] + SPP[3,ST] * ( COS( 2*I * (ST+1)*UPBD/M ) -
                                                COS( 2*I * ST*UPBD/M ) );

       FOUKO[I] := - FOUKO[I] / ( 4*I*I ) - 2 * SPP[1,0];
       FOUKO[I] := FOUKO[I] / ( UPBD * 2 * I * I );

   END;    

   SFOUKOEFF := FOUKO;    

END;          
          
(*--------------------------------------------------------------------------*)

END.

(*-------------------------------------------------------------------------*)
(*                                                                         *)
(*  PASCAL-XSC - MODUL  SPLINES                                            *)
(*                                                                         *)
(*  KONSTRUIERT AUS DEN FUNKTIONSWERTEN EINER SYMMETRISCHEN FUNKTION EINEN *)
(*  SYMMETRISCHEN, KUBISCHEN, ZWEIMAL STETIG DIFFERENZIERBAREN SPLINE      *)
(*                                                                         *)
(*-------------------------------------------------------------------------*)
